/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_CAMERADRIVINGSERVICE_CAMERADRIVINGSERVICE_SKELETON_H
#define JAC_AP_CAMERADRIVINGSERVICE_CAMERADRIVINGSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/cameradrivingservice/cameradrivingservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace cameradrivingservice {
namespace skeleton {
namespace events
{
    using CamDrv2CamDrvPerceptionOneEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::Cam2CamPerceptionStruct>;
    using CamDrv2CamDrvPerceptionTwoEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::Cam2CamPerceptionStruct>;
    using CamDrv2CamDrvPerceptionThreeEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::Cam2CamPerceptionStruct>;
    using CamDrv2CamDrvPerceptionFourEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::Cam2CamPerceptionStruct>;
    static constexpr ara::com::internal::EntityId CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId = 30182U; //CamDrv2CamDrvPerceptionOneEvent_event_hash
    static constexpr ara::com::internal::EntityId CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId = 28892U; //CamDrv2CamDrvPerceptionTwoEvent_event_hash
    static constexpr ara::com::internal::EntityId CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId = 5485U; //CamDrv2CamDrvPerceptionThreeEvent_event_hash
    static constexpr ara::com::internal::EntityId CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId = 50588U; //CamDrv2CamDrvPerceptionFourEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class CameraDrivingServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultCamDrv2CamDrvPerceptionOneEvent = CamDrv2CamDrvPerceptionOneEvent.Initialize();
        ThrowError(resultCamDrv2CamDrvPerceptionOneEvent);
        const ara::core::Result<void> resultCamDrv2CamDrvPerceptionTwoEvent = CamDrv2CamDrvPerceptionTwoEvent.Initialize();
        ThrowError(resultCamDrv2CamDrvPerceptionTwoEvent);
        const ara::core::Result<void> resultCamDrv2CamDrvPerceptionThreeEvent = CamDrv2CamDrvPerceptionThreeEvent.Initialize();
        ThrowError(resultCamDrv2CamDrvPerceptionThreeEvent);
        const ara::core::Result<void> resultCamDrv2CamDrvPerceptionFourEvent = CamDrv2CamDrvPerceptionFourEvent.Initialize();
        ThrowError(resultCamDrv2CamDrvPerceptionFourEvent);
    }

    CameraDrivingServiceSkeleton& operator=(const CameraDrivingServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId, mode)),
              CamDrv2CamDrvPerceptionOneEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId),
              CamDrv2CamDrvPerceptionTwoEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId),
              CamDrv2CamDrvPerceptionThreeEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId),
              CamDrv2CamDrvPerceptionFourEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec, mode)),
              CamDrv2CamDrvPerceptionOneEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec),
              CamDrv2CamDrvPerceptionTwoEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec),
              CamDrv2CamDrvPerceptionThreeEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec),
              CamDrv2CamDrvPerceptionFourEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer, mode)),
              CamDrv2CamDrvPerceptionOneEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer),
              CamDrv2CamDrvPerceptionTwoEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer),
              CamDrv2CamDrvPerceptionThreeEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer),
              CamDrv2CamDrvPerceptionFourEvent(ptr->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              CamDrv2CamDrvPerceptionOneEvent(std::move(other.CamDrv2CamDrvPerceptionOneEvent)),
              CamDrv2CamDrvPerceptionTwoEvent(std::move(other.CamDrv2CamDrvPerceptionTwoEvent)),
              CamDrv2CamDrvPerceptionThreeEvent(std::move(other.CamDrv2CamDrvPerceptionThreeEvent)),
              CamDrv2CamDrvPerceptionFourEvent(std::move(other.CamDrv2CamDrvPerceptionFourEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                CamDrv2CamDrvPerceptionOneEvent = std::move(other.CamDrv2CamDrvPerceptionOneEvent);
                CamDrv2CamDrvPerceptionTwoEvent = std::move(other.CamDrv2CamDrvPerceptionTwoEvent);
                CamDrv2CamDrvPerceptionThreeEvent = std::move(other.CamDrv2CamDrvPerceptionThreeEvent);
                CamDrv2CamDrvPerceptionFourEvent = std::move(other.CamDrv2CamDrvPerceptionFourEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::CamDrv2CamDrvPerceptionOneEvent GetCamDrv2CamDrvPerceptionOneEvent()
        {
            return std::move(CamDrv2CamDrvPerceptionOneEvent);
        }
        events::CamDrv2CamDrvPerceptionTwoEvent GetCamDrv2CamDrvPerceptionTwoEvent()
        {
            return std::move(CamDrv2CamDrvPerceptionTwoEvent);
        }
        events::CamDrv2CamDrvPerceptionThreeEvent GetCamDrv2CamDrvPerceptionThreeEvent()
        {
            return std::move(CamDrv2CamDrvPerceptionThreeEvent);
        }
        events::CamDrv2CamDrvPerceptionFourEvent GetCamDrv2CamDrvPerceptionFourEvent()
        {
            return std::move(CamDrv2CamDrvPerceptionFourEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = CamDrv2CamDrvPerceptionOneEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = CamDrv2CamDrvPerceptionTwoEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = CamDrv2CamDrvPerceptionThreeEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = CamDrv2CamDrvPerceptionFourEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::CamDrv2CamDrvPerceptionOneEvent CamDrv2CamDrvPerceptionOneEvent;
        events::CamDrv2CamDrvPerceptionTwoEvent CamDrv2CamDrvPerceptionTwoEvent;
        events::CamDrv2CamDrvPerceptionThreeEvent CamDrv2CamDrvPerceptionThreeEvent;
        events::CamDrv2CamDrvPerceptionFourEvent CamDrv2CamDrvPerceptionFourEvent;
    };
    explicit CameraDrivingServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId, mode)),
          CamDrv2CamDrvPerceptionOneEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId),
          CamDrv2CamDrvPerceptionTwoEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId),
          CamDrv2CamDrvPerceptionThreeEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId),
          CamDrv2CamDrvPerceptionFourEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit CameraDrivingServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec, mode)),
          CamDrv2CamDrvPerceptionOneEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec),
          CamDrv2CamDrvPerceptionTwoEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec),
          CamDrv2CamDrvPerceptionThreeEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec),
          CamDrv2CamDrvPerceptionFourEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit CameraDrivingServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer, mode)),
          CamDrv2CamDrvPerceptionOneEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionOneEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer),
          CamDrv2CamDrvPerceptionTwoEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionTwoEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer),
          CamDrv2CamDrvPerceptionThreeEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionThreeEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer),
          CamDrv2CamDrvPerceptionFourEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingServiceCamDrv2CamDrvPerceptionFourEventId, ::jac::ap::cameradrivingservice::CameraDrivingService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    CameraDrivingServiceSkeleton(const CameraDrivingServiceSkeleton&) = delete;

    CameraDrivingServiceSkeleton(CameraDrivingServiceSkeleton&&) = default;
    CameraDrivingServiceSkeleton& operator=(CameraDrivingServiceSkeleton&&) = default;
    CameraDrivingServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          CamDrv2CamDrvPerceptionOneEvent(token.GetCamDrv2CamDrvPerceptionOneEvent()),
          CamDrv2CamDrvPerceptionTwoEvent(token.GetCamDrv2CamDrvPerceptionTwoEvent()),
          CamDrv2CamDrvPerceptionThreeEvent(token.GetCamDrv2CamDrvPerceptionThreeEvent()),
          CamDrv2CamDrvPerceptionFourEvent(token.GetCamDrv2CamDrvPerceptionFourEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~CameraDrivingServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&CameraDrivingServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::CamDrv2CamDrvPerceptionOneEvent CamDrv2CamDrvPerceptionOneEvent;
    events::CamDrv2CamDrvPerceptionTwoEvent CamDrv2CamDrvPerceptionTwoEvent;
    events::CamDrv2CamDrvPerceptionThreeEvent CamDrv2CamDrvPerceptionThreeEvent;
    events::CamDrv2CamDrvPerceptionFourEvent CamDrv2CamDrvPerceptionFourEvent;
};
} // namespace skeleton
} // namespace cameradrivingservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_CAMERADRIVINGSERVICE_CAMERADRIVINGSERVICE_SKELETON_H
